### This file is prepared for JCGS with R package bst, version 0.3-14
### simulation study for example 1-3 in Quadratic Majorization for Nonconvex Loss with Applications to the Boosting Algorithm, Zhu Wang, 2017 (the paper).
### pdf files of plots similar to Figure 4 in the paper are generated 

###-------------- ONLY change the following two lines to replicate the simulation study in the paper --------------------
ex <- 3  ### Example 1 (ex=1), Example 2 (ex=2) and Example 3 (ex=3) in the paper
rfk <- 11 ### index indicating which loss (and robust loss) is used, see next three lines for the corresponding labels. For instance, rfk=1 means "ClossBoost" in bst function and "ClossBoostQM" in rbst function, respectively. Some labels are not described in the paper but somewhere else (see help(rbst)) and can be ignored below. For Example 1 and 2, rfk = 1-7; for Example 3, rfk =8-11
###-------------- END of change --------------------------------------------------------------------------------------------------

rf <- c("closs", "gloss", "qloss", "texpo", "thinge", "tbinom", "binomd", "clossR", "tgaussian", "thuber", "tgaussian")[rfk]          #loss used in rbst function
bsttype1 <- c("ClossBoost", "GlossBoost", "QlossBoost", "AdaBoost", "HingeBoost", "LogitBoost", "LogitBoost", "ClossRBoost", "L2Boost", "HuberBoost", "L1Boost") #label of loss in bst function
rbsttype1 <- c("ClossBoostQM", "GlossBoostQM", "QlossBoostQM", "TAdaBoost", "THingeBoost", "TLogitBoost", "DLogitBoost", "ClossRBoostQM", "TGaussian", "THuberBoost", "TGaussian") #label of loss used in rbst function

### --------------configuration 1----------------
### begin of user configurations for data generation
nrun <- 100 ### number of Monte Carlo simulations
ntr <- ntu <- 200 ### number of training (ntr) and tuning (ntu) data
nte <- 10000      ### number of test data
if(ex %in% c(1, 2)) p <- 18 else p <- 8 #number of non-effective predictor variables
per <- c(0, 5, 10, 20)/100  ### Outliers are randomly introduced to percentage of v=0, 5, 10 and 20
### end of user configurations for data generation ###

### --------------configuration 2----------------
### begin of user configurations in data modeling

### s is the sigma in the paper, four s values corresponding to specified per above
nu <- 0.1
if(ex==1){
    if(rf=="closs") s <- rep(0.9, 4) 
    else if(rf=="gloss") s <- c(2, 2, 1.5, 1.3) 
    else if(rf=="qloss") s <- c(0.1, 0.1, 0.3, 0.3)
    else if(rf=="texpo") s <- rep(-log(2), 4) ### experimental value
    else if(rf=="thinge") s <- rep(-2, 4) ### experimental value
    else if(rf=="tbinom") s <- rep(-log(3), 4) ### experimental value
    else if(rf=="binomd") s <- rep(log(4), 4) ### experimental value
    if(rf=="qloss") nu <- 0.01 
}else 
    if(ex==2){
        if(rf=="closs") s <- c(1, 1, 1, 1.4)
        else if(rf=="gloss") s <- rep(1.5, 4)
        else if(rf=="qloss") s <- rep(0.5, 4) 
        else if(rf=="texpo") s <- rep(-log(2), 4) ### experimental value
        else if(rf=="thinge") s <- rep(-2, 4) ### experimental value
        else if(rf=="tbinom") s <- rep(-log(3), 4) ### experimental value
        else if(rf=="binomd") s <- rep(log(4), 4) ### experimental value
	if(rf=="qloss") nu <- 0.02 
    }else
        if(ex==3){
            s <- rep(3, 4)
            if(rf=="clossR"){
                nu <- 0.5
            }
        }
if(rf=="gloss" && s <=1) stop("s must be > 1\n")
if(ex %in% c(1, 3)) learner <- "ls" else learner <- c("sm", "tree")[1]
ms <- 200  ### maximum boosting iteration 
if(ex==1 && rf %in% c("tbinom", "binomd")){
    ms <- 400
}else if(ex==2 && rf %in% c("tbinom", "binomd")){
    ms <- 600
}else if(ex==2 && rf=="thinge") {ms <- 200
}else if(ex==2 && rf=="texpo") ms <- 100
### end of user configurations in data modeling ###

### Set up simulations
summary7 <- function(x, na.rm=TRUE) c(summary(x, na.rm=na.rm), sd=sd(x, na.rm=na.rm))
library("mvtnorm")
rho <- 0.0
if(!ex %in% 1:3) stop("Not implemented\n")
datagen <- function(ntr, ntu, nte, ex, percon, rf, p=18, sigma=NULL){
    if(ex==1 || ex==2)
        if(rf=="tgaussian" || rf=="tpoisson") stop("example data and family not match\n") 
    n <- ntr+ntu+nte
    if(ex==1 || ex==2){
        r <- runif(n, 0, 1)
        theta <- runif(n, 0, 2*pi)
        x1 <- sqrt(r)*cos(theta)
        x2 <- sqrt(r)*sin(theta)
        if(ex==1)
            y <- ifelse(x1 >= x2, 1, -1) ### Example 1 in Park and Liu (2011)
        else y <- ifelse((x1-x2)*(x1+x2) < 0, 1, -1) ### Example 2 in Park and Liu (2011)
        xnois <- matrix(runif(n*p, -1, 1), ncol=p)
        if(p > 0)
            x <- cbind(x1, x2, xnois) 
        else x <- cbind(x1, x2)
        xnonzero <- 1:2
        if(p > 0)
            xzero <- 3:(p+2)
        else xzero <- 0
        d <- p+2
        b <- NA
        A <- NA
    }
    else if(ex==3){
        corrtype <- 1
        rho <- 0.5
        sigma2 <- 3
	b <- c(3, 1.5, 0, 0, 2, 0, 0, 0)
        d1 <- length(b)
        d <- p
        b <- c(b, rep(0, p-d1))
        xnonzero <- which(b!=0)
        xzero <- which(b==0)
        A <- matrix(rep(NA*d*d),d,d)
        for(i in 1:d)
            for(j in 1:d){
                if(corrtype==1)
                    A[i,j] <- rho^abs(i-j)
                else A[i,j] <- rho
            }
        if(corrtype==2) diag(A) <- 1
        x <- rmvnorm(n,rep(0,d),sigma=A)
        yclean <- y <- x%*%b
        y <- y+rnorm(n, 0, sd=sqrt(sigma2))
    }
    x <- as.data.frame(x)
    xtr <- x[1:ntr,]
    ytr <- y[1:ntr]
    xtu <- x[(1+ntr):(ntr+ntu),]
    ytu <- y[(1+ntr):(ntr+ntu)]
    xte <- x[(1+ntr+ntu):n,]
    yte <- y[(1+ntr+ntu):n]
    con <- sample(ntr) 
    conte <- sample(nte) 
    ytrold <- ytr
    ytuold <- ytu
    yteold <- yte
    if(percon > 0){ ### to generate contaminated data
        j <- con[1:(percon*ntr)]
        jte <- conte[1:(percon*nte)]
        if(ex %in% c(1, 2)){
            ytr[j] <- -ytr[j]
            ytu[j] <- -ytu[j]
            yte[jte] <- -yte[jte]
        }
        else if(ex==3){
            ytr[j] <- 1000*ytr[j]
            ytu[j] <- 1000*ytu[j]
        }
    }
    RET <- list(xtr=xtr, ytr=ytr, xtu=xtu, ytu=ytu, xte=xte, yte=yte, ytrold=ytrold, ytuold=ytuold, yteold=yteold, d=d, xnonzero=xnonzero, xzero=xzero, b=b, A=A)
}
n <- ntr+ntu+nte
### End of data generation 


### Predictive modeling
library("bst")
if(ex==3 && rfk <= 7) stop("For example 3, rfk must be >= 8\n")
twin <- FALSE
if(rf=="closs") trun <- TRUE else trun <- FALSE  ### to truncate the predict values in boosting. It seems to work better for closs in classification problems.
if(rf!="closs" && trun) stop("check if you really want to set trun=TRUE\n")
tuning <- "error"
bstfam <- rf
if(rf=="texpo"){
    bstfam <- "expo"
} else if(rf=="thinge"){ 
    bstfam <- "hinge"
}else if(rf=="tbinom"){ 
    bstfam <- "binom"
}else if(rf=="binomd"){ 
    bstfam <- "binom"
}else if(rf=="tgaussian"){ 
    bstfam <- "gaussian"
}else if(rf=="thuber") {bstfam <- "huber"}
if(rfk==11) bstfam <- "lar"  ###absolute difference loss (L_1 loss)
for(k in (1:length(per))[1:4]){  ### k controls which contamination data percentage
    nmethod <- 3
    err <- errold <- sen <- spc <- matrix(NA, ncol=nmethod, nrow=nrun)
    colnames(err) <- colnames(errold) <- colnames(sen) <- colnames(spc) <- c(paste(bsttype1[rfk], "(clean data)"), bsttype1[rfk], rbsttype1[rfk])
    nvar.m <- nvar.m2 <- err.m <- err.m2 <- loss.m <- loss.m2 <- matrix(NA, ncol=ms, nrow=nrun)
    mstop1 <- mstop2 <- optqu <- s.sel <- ptm.m2 <- ptm.m <- rep(NA, nrun)
    for(i in 1:nrun){
        if(ex!=4)
            cat("percentage of contamination", per[k], "\n")
        cat("i=", i, "\n")
        set.seed(1000+i)
### generate uniform distribution over a unit disk, cf: Wu and Liu (2007), Park and Liu (2011), http://mathworld.wolfram.com/DiskPointPicking.html
        if(ex <= 3) dat <- datagen(ntr, ntu, nte, ex=ex, percon=per[k], p=p, rf=rf)
        xzero <- dat$xzero
        xnonzero <- dat$xnonzero
        xtr <- dat$xtr
        xtu <- dat$xtu
        xte <- dat$xte
        ytr <- dat$ytr
        ytu <- dat$ytu
        yte <- dat$yte
        ytrold <- dat$ytrold
        ytuold <- dat$ytuold
        yteold <- dat$yteold
        b <- dat$b
        d <- dat$d
        A <- dat$A
	
### rbst computing
        mstop <- ms
        err1 <- rep(NA, length(s))
###outloop on s
        fit <- vector("list", length(s))
	ks <- k
                                        #for(ks in 1:length(s)){
        ptm <- proc.time()
        fit[[ks]] <- rbstpath(xtr, ytr, rmstop=mstop, ctrl = bst_control(nu=nu, s=s[ks], sh=s[ks]/2, trace=FALSE, center=FALSE, iter=100, trun=trun), rfamily = rf, learner = learner, del=1e-20)
        ptm.m2[i] <- (proc.time()-ptm)[2]
        if(twin)
            fit[[ks]] <- rbstpath(xtr, ytr, rmstop=mstop, ctrl = bst_control(nu=nu, s=s[ks], twinboost=TRUE, center=FALSE, iter=100, trun=trun, f.init=predict(fit[[ks]][[1]]), xselect.init = fit[[ks]][[1]]$xselect),rfamily = rf, learner = learner, del=1e-16)
        res2 <- res3 <- rep(NA, length(mstop))
        for(jk in 1:length(mstop)){
            res1 <- predict(fit[[ks]][[jk]], newdata=xtu, newy=ytu, type=tuning)
            res2[jk] <- min(res1)
        }
### for fixed s, find the best model
        dat.m2 <- fit[[ks]][[which.min(res2)]]
        res1 <- predict(dat.m2, newdata=xtu, newy=ytu, type=tuning)
        res <- predict(dat.m2, newdata=xte, newy=yte, type="error")
        resold <- predict(dat.m2, newdata=xte, newy=yteold, type=tuning)
        if(p <= 2) minm <- mstop[length(mstop)]
        else
            if(p < ntr)
                minm <- max(1, which.min(res1))
        else minm <- max(10, which.min(res1))
        err1[ks] <- res1[minm]
        cat("\ns=", s[ks], "minimum error=", err1[ks], "\n")    
                                        #}
###find smallest error among different s
        findmin <- which.min(err1)
                                        #        if(rf=="closs" & per[k] %in% c(0, 0.05))
### if there are ties, use the largest value as s, thus less convex, or smaller number of outliers. Note, for Qloss, this should be opposite direction
                                        #            findmin <- max(which(err1==min(err1)))  
	dat.m2 <- fit[[findmin]]
### for the best s, find the best model (or best mstop)
        res2 <- rep(NA, length(mstop))
        for(jk in 1:length(mstop)){
            res1 <- predict(dat.m2[[jk]], newdata=xtu, newy=ytu, type=tuning)
            res2[jk] <- min(res1)
        }
        optqu[i] <- findmin
        cat("\n method in rbst: ", rf)
        cat("\n method in bst: ", bstfam)
        cat("\n nu=", nu, "\n")
                                        #cat("\n which s has smallest error/loss", s[findmin], "\n")
        s.sel[i] <- s[findmin]
###find smallest error among different mstop
        dat.m2 <- dat.m2[[which.min(res2)]]
        res1 <- predict(dat.m2, newdata=xtu, newy=ytu, type=tuning)
        res <- predict(dat.m2, newdata=xte, newy=yte, type="error")
        loss.m2[i,] <- dat.m2$risk
	if(ex %in% c(1, 2))
            err.m2[i,] <- res
        else if(ex==3){
            for(jjj in 1: dat.m2$ctrl$mstop){
                coef.m <- coef(dat.m2, which=jjj)
                err.m2[i, jjj] <- t(coef.m-b) %*% A %*% (coef.m-b)
            }
        }
	for(jjj in 1: dat.m2$ctrl$mstop)
            nvar.m2[i,jjj] <- length(unique(dat.m2$ensemble[1:jjj]))
        if(ex!=5)
            mstop1[i] <- mstop[which.min(res2)]
        if(p <= 2) minm <- mstop[length(mstop)]
        else
            if(p < ntr)
                minm <- max(1, which.min(res1))
        else minm <- max(10, which.min(res1))
        mstop2[i] <- minm
	err[i,3] <- res[minm]
	if(ex==3){
            coef.m <- coef(dat.m2, which=minm)
            err[i,3] <- t(coef.m-b) %*% A %*% (coef.m-b)
        }
        sen[i,3] <- sum(xnonzero %in% unique(dat.m2$ensemble[1:minm]))/length(xnonzero)
        spc[i,3] <- sum(!xzero %in% unique(dat.m2$ensemble[1:minm]))/length(xzero)

### bst computing for clean data, i.e, no outliers introduced 
        if(rf!="thuber")
            dat.m0 <- bst(xtr, ytrold, ctrl = bst_control(mstop=ms, center=FALSE, trace=FALSE, trun=trun, nu=nu, s=s[findmin]), family = bstfam, learner = learner)
	else dat.m0 <- bst(xtr, ytrold, ctrl = bst_control(mstop=ms, center=FALSE, trace=FALSE, nu=nu, s=NULL), family = bstfam, learner = learner)
        res1 <- predict(dat.m0, newdata=xtu, newy=ytuold, type=tuning)
        res <- predict(dat.m0, newdata=xte, newy=yteold, type="error")
	resold <- predict(dat.m0, newdata=xte, newy=yteold, type=tuning)
        err[i,1] <- res[which.min(res1)]
	if(ex==3){
            coef.m <- coef(dat.m0, which=which.min(res1))
            err[i,1] <- t(coef.m-b) %*% A %*% (coef.m-b)
        }
        sen[i,1] <- sum(xnonzero %in% unique(dat.m0$ensemble[1:which.min(res1)]))/length(xnonzero)
        spc[i,1] <- sum(!xzero %in% unique(dat.m0$ensemble[1:which.min(res1)]))/length(xzero)
### bst computing for contaminated data
	if(bstfam=="huber")
            dat.m <- bst(xtr, ytr, ctrl = bst_control(mstop=ms, center=FALSE, s=NULL, nu=nu, trace=FALSE), family = bstfam, learner = learner)
	else dat.m <- bst(xtr, ytr, ctrl = bst_control(mstop=ms, center=FALSE, trun=trun, s=s[findmin], nu=nu, trace=FALSE), family = bstfam, learner = learner)
        loss.m[i,] <- dat.m$risk
        if(twin)
            dat.m <- bst(xtr, ytr, ctrl = bst_control(mstop=ms, twinboost=TRUE, center=FALSE, trun=trun, f.init=predict(dat.m), xselect.init = dat.m$xselect, s=s[findmin], nu=nu), family = bstfam, learner = learner)
        res1 <- predict(dat.m, newdata=xtu, newy=ytu, type=tuning)
        res <- predict(dat.m, newdata=xte, newy=yte, type="error")
        err[i,2] <- res[which.min(res1)]
	if(ex==3){
            coef.m <- coef(dat.m, which=which.min(res1))
            err[i,2] <- t(coef.m-b) %*% A %*% (coef.m-b)
        }
        if(ex %in% c(1, 2))
            err.m[i,] <- res
        else if(ex==3){
            for(jjj in 1: dat.m$ctrl$mstop){
                coef.m <- coef(dat.m, which=jjj)
                err.m[i, jjj] <- t(coef.m-b) %*% A %*% (coef.m-b)
            }
        }
### sensitivity
        sen[i,2] <- sum(xnonzero %in% unique(dat.m$ensemble[1:which.min(res1)]))/length(xnonzero)
        spc[i,2] <- sum(!xzero %in% unique(dat.m$ensemble[1:which.min(res1)]))/length(xzero)
        for(jjj in 1: dat.m$ctrl$mstop)
            nvar.m[i,jjj] <- length(unique(dat.m$ensemble[1:jjj]))
        check <- 0
        if(check==1){
            par(mfrow=c(3, 2))
            plot(predict(dat.m2, newdata=xtr, newy=ytr, type="error"), type="l", main="Error of training data", col="red")
            points(predict(dat.m, newdata=xtr, newy=ytr, type="error"), type="l", col="black")
            legend("topright", c("GB", "QMBA"), lty=c("solid", "dashed"), col=c("black", "red"))
            plot(predict(dat.m2, newdata=xtr, newy=ytr, type="loss"), type="l", main="Loss of training data", col="red")
            points(predict(dat.m, newdata=xtr, newy=ytr, type="loss"), type="l", col="black")
            legend("topright", c("GB", "QMBA"), lty=c("solid", "dashed"), col=c("black", "red"))
            plot(predict(dat.m2, newdata=xtu, newy=ytu, type="error"), type="l", main="Error of tuning data", col="red")
            points(predict(dat.m, newdata=xtu, newy=ytu, type="error"), type="l", col="black")
            legend("topright", c("GB", "QMBA"), lty=c("solid", "dashed"), col=c("black", "red"))
            plot(predict(dat.m2, newdata=xtu, newy=ytu, type="loss"), type="l", main="Loss of tuning data", col="red")
            points(predict(dat.m, newdata=xtu, newy=ytu, type="loss"), type="l", col="black")
            legend("topright", c("GB", "QMBA"), lty=c("solid", "dashed"), col=c("black", "red"))
            plot(predict(dat.m2, newdata=xte, newy=yte, type="error"), type="l", col="red", main="Error of test data")
            points(predict(dat.m, newdata=xte, newy=yte, type="error"), type="l", col="black")
            legend("topright", c("GB", "QMBA"), lty=c("solid", "dashed"), col=c("black", "red"))
            plot(predict(dat.m2, newdata=xte, newy=yte, type="loss"), type="l", col="red", main="Loss of test data")
            points(predict(dat.m, newdata=xte, newy=yte, type="loss"), type="l", col="black", ylim=c(0.20, 0.25))
            legend("topright", c("GB", "QMBA"), lty=c("solid", "dashed"), col=c("black", "red"))
        }
        if(i %% 5==0){
                                        #cat("s selected\n")
                                        #print(table(s.sel))
                                        #cat("optimal mstop selected by tuning data in different M\n")
                                        #print(summary(mstop1))
                                        #cat("mstop selected by tuning data\n")
                                        #print(summary(mstop2))
            if(ex %in% 1:2){
                cat("\nMisclassification error\n")
                print(apply(err, 2, summary7, na.rm=TRUE))
            }   
            else if(ex==3){
                cat("\nMean squared prediction error\n")
                print(apply(err, 2, summary7, na.rm=TRUE))
            }
                                        #cat("\nsensitivity\n")
                                        #print(apply(sen, 2, summary, na.rm=TRUE))
                                        #cat("\nspecificity\n")
                                        #print(apply(spc, 2, summary, na.rm=TRUE))
            cat("\nNumber of selected variables\n")
            print(apply((1-spc)*length(xzero)+sen*length(xnonzero), 2, summary7, na.rm=TRUE))
        }
	nvar <- (1-spc)*length(xzero)+sen*length(xnonzero)
        mu.loss.m <- apply(loss.m, 2, mean, na.rm=TRUE)
        se.loss.m <- apply(loss.m, 2, sd, na.rm=TRUE)/sqrt(i) ### standard error of mean
        mu.loss.m2 <- apply(loss.m2, 2, mean, na.rm=TRUE)
        se.loss.m2 <- apply(loss.m2, 2, sd, na.rm=TRUE)/sqrt(i)
        mu.err.m <- apply(err.m, 2, mean, na.rm=TRUE)
        mu.err.m2 <- apply(err.m2, 2, mean, na.rm=TRUE)
	mu.nvar.m <- apply(nvar.m, 2, mean, na.rm=TRUE)
        mu.nvar.m2 <- apply(nvar.m2, 2, mean, na.rm=TRUE)
        plotloss <- function(){
            par(mfrow=c(1, 3))
    	    ylim1 <- min(c(mu.loss.m2-1.965*se.loss.m2, mu.loss.m-1.965*se.loss.m), na.rm=TRUE)
            ylim2 <- max(c(mu.loss.m2+1.965*se.loss.m2, mu.loss.m+1.965*se.loss.m), na.rm=TRUE)
            plot(mu.loss.m, type="l", main="", ylab="Loss value", xlab="Iteration", ylim=c(ylim1, ylim2))
            points(mu.loss.m-1.965*se.loss.m, type="l", lty="dotted")
            points(mu.loss.m+1.965*se.loss.m, type="l", lty="dotted")
            points(mu.loss.m2, type="l", col="red", lty="dashed")
            points(mu.loss.m2-1.965*se.loss.m2, type="l", col="red", lty="dotted")
            points(mu.loss.m2+1.965*se.loss.m2, type="l", col="red", lty="dotted")
            legend("topright", c(bsttype1[rfk], rbsttype1[rfk]), col=c("black", "red"), lty=c("solid", "dashed")) 
            plot(mu.err.m, type="l", main="", ylab="Misc error", xlab="Iteration")
            points(mu.err.m2, type="l", main="", ylab="Misc error", xlab="Iteration", col="red", lty="dashed")
            legend("topright", c(bsttype1[rfk], rbsttype1[rfk]), col=c("black", "red"), lty=c("solid", "dashed")) 
            plot(mu.nvar.m, type="l", main="", col="blue", ylab="No. variables", xlab="Iteration")
            points(mu.nvar.m2, type="l", main="", ylab="No. variables", xlab="Iteration", col="red", lty="dashed")
            legend("bottomright", c(bsttype1[rfk], rbsttype1[rfk]), col=c("black", "red"), lty=c("solid", "dashed")) 
        }
	if(i == nrun){
            pdf(file=paste("ex", ex, "p", d, "con", k,"_rf", rf, "_twin", twin, "_s", abs(s[k]), "loss.pdf", sep=""))       
            plotloss()
	    dev.off()
        }
        res <- list(contscen=k, p=d, bstfam=bstfam, rf=rf, s=s, nu=nu, err=err, errold=errold, sen=sen, spc=spc, mstop2=mstop2, s.sel=s.sel, optqu=optqu, nvar=nvar, err.m=err.m, err.m2=err.m2, nvar.m=nvar.m, nvar.m2=nvar.m2, loss.m=loss.m, mu.loss.m=mu.loss.m, se.loss.m=se.loss.m, loss.m2=loss.m2, mu.loss.m2=mu.loss.m2, se.loss.m2=se.loss.m2, ptm.m2=ptm.m2)
        if(i == nrun){
            ploterr <- function(){
                if(ex==3) yla <- "Mean squared prediction error" else yla <- "Misclassification error"
                par(mfrow=c(2, 1))
                if(ms==200){
                    boxplot(cbind(res$err.m[, 50], res$err.m2[,50], res$err.m[, 100], res$err.m2[,100], res$err.m[, 150], res$err.m2[, 150], res$err.m[,200], res$err.m2[, 200]), names=c("B50", "RB50", "B100", "RB100", "B150", "RB150", "B200", "RB200"), col=rep(c("blue", "red"), 4), lty=rep(c(1, 2), 4), ylab=yla)
                    boxplot(cbind(res$nvar.m[, 50], res$nvar.m2[,50], res$nvar.m[, 100], res$nvar.m2[,100], res$nvar.m[, 150], res$nvar.m2[, 150], res$nvar.m[, 200], res$nvar.m2[, 200]), names=c("B50", "RB50", "B100", "RB100", "B150", "RB150", "B200", "RB200"), col=rep(c("blue", "red"), 4), lty=rep(c(1, 2), 4), ylab="No. Variables")
                }
            }
        }
    }
}
## ----sessionInfo---------------------------------------------------------
sessionInfo();
q("no")
